#ifndef __CBASIS_IMMIX_BLOCK_POOL_H__
#define __CBASIS_IMMIX_BLOCK_POOL_H__

// Clear the line bitmap and the block list pointer
void IbpInitImmixBlock(void *block);

void *IbpGetImmixBlockAreaStart(void *block);

void *IbpGetImmixBlockAreaEnd(void *block);

void *IbpGetImmixBlockListPtr(void *block);

void IbpSetImmixBlockListPtr(void *block, void *ptr);

// This is like a Standard ML list cons, where blockPool is a pointer to
// the first block in the list and block is the new block to be cons'ed
void *IbpInsertImmixBlock(void *block, void *blockPool);

// Allocate initialize and put some blocks on the free list
void *IbpAllocFreshBlocks();

void *IbpGetFreeBlock();

// Find a block and an allocation span with at least bytes free bytes available
void IbpGetBlockWithFreeBytes(int bytes);

// currentPtr is about to pass endPtr, find a new allocation span
void IbpGetFreeAllocationSpan(int bytes);

// The final managed allocation function
void *IbpAlloc(int bytes);

// Clear all the line mark bitmaps of blocks which are in use
void IbpClearLineMarkBitmapsInUse(unsigned int clearMask);

// The function that garbage collection is all about
void IbpReclaimGarbage();

// Returns 1 if we should reclaim garbage, 0 otherwise
int IbpShouldReclaimGarbage();

// Return the tracing-function clear-mask for either a full
// or a partial heap collection, depending on what it is time for
unsigned int IbpFullOrPartialClearMask();

// Returns 1 if we should continue tracing back upwards the stack, 0 otherwise
int IbpShouldStillTrace();

// Mark a line bit corresponding to a given address
void IbpLineMarkAddr(void *addr);

#endif // __CBASIS_IMMIX_BLOCK_POOL_H__
