#include "CBasisInternalHeapMng.h"
#include <fcntl.h>    // For O_RDONLY
#include <unistd.h>   // For getpagesize
#include <sys/mman.h> // For mmap and munmap
#include <stdio.h>
#include <stdlib.h>

// Compute an address aligned to the given size, which must be a power of two.
// The resulting address will always be after the given address,
// up to powTwoSize bytes after, in case it happened to be aligned
char *IhmAlignAddrToPowTwoSize(char *addr, unsigned int powTwoSize) {
  return ((char *)(((unsigned int)addr) & (~(powTwoSize - 1)))) + powTwoSize;
}

// Allocate a given number of contiguous blocks, aligned to block size
void *IhmAllocAlignedBlocks(unsigned int blockCnt) {
  // Allocate blockCnt blocks of memory, plus one extra block for alignment.
  // According to documentation, mmap aligns to page size, which is usually
  // 4096 bytes
  int fd = open("/dev/zero", O_RDONLY);
  //puts("Opened device");
  unsigned int bytes = IHM_BLOCK_SIZE * (blockCnt + 1);
  char *addr = mmap(NULL, bytes, PROT_WRITE | PROT_READ, MAP_PRIVATE, fd, 0);
  //printf("addr: %d\n", (int)addr);
  close(fd);

  if(!addr) {
      puts("System memory exhausted. Exiting to avoid crashes");
      exit(-1);
  }
  
  // Compute an adjusted address aligned to the block size.
  // It will always be after the given address, so there is always room before
  char *alignedAddr = IhmAlignAddrToPowTwoSize(addr, IHM_BLOCK_SIZE);
  //printf("alignedAddr: %d\n", (int)alignedAddr);
  
  if(((unsigned int)(alignedAddr - addr)) < 32) {
    puts("The allocated memory area was not aligned to at least 32 bytes, as expected. Exiting to avoid crashes");
    exit(-1);
  }
  
  // Use four bytes of the room before alignedAddr to remember
  // the allocated address and size, for the munmap later.
  // This part should scale to 64-bit memory management
  *((unsigned int *)(alignedAddr - 8)) = bytes;
  *((char **)(alignedAddr - 16)) = addr;
  
  return (void *)alignedAddr;
}

// Free a set of contiguous blocks allocated by IhmAllocAlignedBlocks
void IhmFreeAlignedBlocks(void *alignedAddr) {
  int bytes = *((unsigned int *)(((char *)alignedAddr) - 8));
  char *addr = *((char **)(((char *)alignedAddr) - 16));

  munmap(addr, bytes);
}

// When executing the code below on Gandalf
// (Ubuntu Linux 8.04 on Intel Pentium Duo Core 2, 2.4Ghz),
// It showed a page size of 4096.
/*
int main() {
  int sz = getpagesize();
  printf("Pagesize: %d\n", (int)sz);
}
*/
